@@ -1,3 +1,23 @@
+/*
+ * Copyright (C) 2001  Roy Keene
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ *      email: dact@rkeene.org
+ */
+
 /*
 	Encrypt data.
 */
@@ -46,8 +66,8 @@ int cipher_sub_init_getkey(const int mode, char *key) {
 	int fd,x=257;
 
 	fname=dact_ui_getuserinput("Key file: ",128,0);
-	if (mode==DACT_MODE_CDEC) {
-		fd=open(fname, O_RDONLY);
+	fd=open(fname, O_RDONLY);
+	if (fd>=0) {
 		x=read(fd, &keybuf, sizeof(keybuf));
 		if (x==257) {
 			memcpy(key,keybuf,257); /* For backward compatability with DACT 0.8.1*/
@@ -58,8 +78,8 @@ int cipher_sub_init_getkey(const int mode, char *key) {
 		close(fd);
 		return(257);
 	}
-	if (mode==DACT_MODE_CENC) {
-		fd=open(fname, O_WRONLY|O_CREAT|O_TRUNC, 0600);
+	if (fd<0 && mode==DACT_MODE_CENC) {
+		fd=open(fname, O_WRONLY, 0600);
 		memcpy(key,generatekey(),257);
 		memcpy(keybuf, ptrbuf=mimes64(key,&x), 400);
 		write(fd, keybuf, x);
