@@ -1,10 +1,32 @@
+/*
+ * Copyright (C) 2001  Roy Keene
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ *      email: dact@rkeene.org
+ */
+
 #define __DACT_C
 #include <fcntl.h>
 #include <stdio.h>
+#include <ctype.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <sys/types.h>
 #include "dact.h"
 #include "parse.h"
@@ -68,7 +90,7 @@ int print_help(int argc, char **argv) {
 	printf("  -m CONF     Load config file CONF.\n");
 	printf("  -u URL      Specify download location as URL.\n");
 	printf("  -p URL      Parse URL and print results, then exit.\n");
-	printf("  -D DESC     Specy a description of DESC.\n");
+	printf("  -D DESC     Specify a description of DESC.\n");
 	printf("  -I NN       Use ONLY 2 algorithms, NN and 0.\n");
 	printf("  -U FILE     Use FILE to select download location.\n");
 	printf("  -E CIPHER   Use CIPHER to encrypt data (LIST  lists available ciphers.)\n");
@@ -153,6 +175,49 @@ int dact_upgrade(const char *options, uint32_t *crcs) {
 	int inFd, outFd;
 	uint32_t i;
 
+
+#ifdef DACT_DEBIAN_UPGRADE_PROC
+	int status=0;
+
+	if (getuid()==0) {
+		buf=dact_ui_getuserinput("Executing `apt-get update' okay [y/N]? ", 5, 0);
+		if (toupper(buf[0])!='Y') {
+			PRINTERR("Failed to upgrade DACT.");
+			free(buf);
+			return(-1);
+		}
+		free(buf);
+
+		if (fork()==0) {
+			execl("/usr/bin/apt-get","apt-get","update",NULL);
+			return(-1);  /* Couldn't run binary. */
+		} else {
+			i=wait(&status);
+			if (WIFEXITED(status)) {
+				if (WEXITSTATUS(status)) {
+					PRINTERR("Failed to run `apt-get update'");
+					return(-1);
+				}
+			}
+		}
+
+		if (fork()==0) {
+			execl("/usr/bin/apt-get","apt-get","install","dact",NULL);
+			return(-1);  /* Couldn't run binary. */
+		} else {
+			i=wait(&status);
+			if (WIFEXITED(status)) {
+				if (WEXITSTATUS(status)) {
+					PRINTERR("Failed to run `apt-get install dact'");
+					return(-1);
+				}
+			}
+		}
+
+		return(1);
+	}
+#endif
+
 	if ((i=is_latest(options))) {
 		PRINTERR("**+");
 		PRINT_LINE; fprintf(stderr, "dact: **> CURR: DACT %i.%i.%i\n",DACT_VER_MAJOR, DACT_VER_MINOR, DACT_VER_REVISION);
@@ -170,6 +235,7 @@ int dact_upgrade(const char *options, uint32_t *crcs) {
 		}
 		fprintf(stderr, "------------------------\n");
 		close(inFd);
+		free(buf);
 	}
 
 #ifdef GO32
@@ -248,8 +314,16 @@ char *dact_getoutfilename(const char *orig, const int mode) {
 	switch (mode) {
 		case DACT_MODE_COMPR:
 			ret=malloc(strlen(orig)+5);
+#ifdef GO32
+			strncpy(ret,orig,8);
+			for (x=0;x<strlen(ret);x++) {
+				if (ret[x]=='.') ret[x]='_';
+			}
+			strcat(ret,".dct");
+#else
 			strcpy(ret,orig);
 			strcat(ret,".dct");
+#endif
 			break;
 		case DACT_MODE_DECMP:
 			if (strcmp(&orig[strlen(orig)-4],".dct") && \
@@ -281,6 +355,10 @@ uint32_t dact_process_other(int src, const int dest, const uint32_t magic, const
 	gzFile gzfd;
 #endif
 
+
+	filesize=0;  /* Fix a warning, that is all. */
+
+
 /*
  * bad and broke stuff XXX FIXME XXX FIXME XXX FIXME
  * There has to be a better way to do this... I just want
@@ -290,12 +368,13 @@ uint32_t dact_process_other(int src, const int dest, const uint32_t magic, const
  * I give up on this.
  *
  * Someone please fix it.
- *  -- Roy Keene <rkeene@rkeene.org>
+ *  -- Roy Keene <dact@rkeene.org>
  */
 	if (lseek_net(src, 0, SEEK_SET)<0) {
 		PRINTERR("This should not be happening.");
 /*
  * lseek_net() should make this obsolete.
+ *  ... EXCEPT! when reading from stdin.
  *
  */
 		tmpfd=mkstemp(tmpbuf);
@@ -538,7 +617,10 @@ uint32_t dact_process_file(const int src, const int dest, const int mode, const
 				dact_ui_status_append(DACT_UI_LVL_GEN,algorithm_names[algo]);
 
 				crcs[0]=ELFCRC(crcs[0], out_buf, retsize);
-				crcs[1]=ELFCRC(crcs[1], in_buf, DACT_BLK_SIZE);
+/* Do not generate a CRC of the plaintext if encrypting */
+				if (cipher==-1) {
+					crcs[1]=ELFCRC(crcs[1], in_buf, DACT_BLK_SIZE);
+				}
 
 				if (!options[DACT_OPT_HDONLY]) {
 					write(dest, &algo, 1);
@@ -714,7 +796,7 @@ XXX: Todo, make this do something...
 */
 		if (filesize==0) {
 /* See if we can rewind our stream, so when we get to the end, we can come back! */
-			if (lseek_net(src, 1, SEEK_SET)==0) {
+			if (lseek_net(src, 1, SEEK_SET)==1) { /* MAJOR BUG HERE! was: lseek(src,1,SEEK_SET)==0 which will always be false. */
 				canlseek=1;
 				lseek_net(src, -8, SEEK_END);
 				read_de(src, &magic, 4);
@@ -808,7 +890,10 @@ XXX: Todo, make this do something...
 			}
 			fileoutsize+=bytes_read;
 
-			crcs[1]=ELFCRC(crcs[1],out_buf,bytes_read);
+/* If ciphering, don't try to calculate this CRC. */
+			if (cipher==-1) {
+				crcs[1]=ELFCRC(crcs[1],out_buf,bytes_read);
+			}
 
 			dact_ui_incrblkcnt(1);
 
@@ -1012,7 +1097,7 @@ int main(int argc, char **argv) {
 		}
 	}
 
-	while ((opt=getopt(argc,argv,"adfsvcnNVHCE:p:I:m:e:lb:hu:U:TPOD:"))!=-1) {
+	while ((opt=getopt(argc,argv,"adfsvcnNVHCE:p:I:m:e:lb:hu:U:TPOD:o:"))!=-1) {
 		switch (opt) {
 			case 'a':
 				PRINTERR("This option does nothing.");
@@ -1057,6 +1142,10 @@ int main(int argc, char **argv) {
 			case 'H':
 				options[DACT_OPT_HDONLY]=!options[DACT_OPT_HDONLY];
 				break;
+			case 'o':
+				out_file=malloc(strlen(optarg)+1);
+				strcpy(out_file,optarg);
+				break;
 			case 'N':
 				return(dact_upgrade(options,crcs));
 				break;
@@ -1064,12 +1153,13 @@ int main(int argc, char **argv) {
 				strtolower(optarg);
 				x=hash_fourbyte(optarg, ' ');
 				if (x==hash_fourbyte("list", ' ')) {
+					PRINT_LINE; fprintf(stderr, "dact: Num | Name\n");
 					for (i=0;i<CIPHER_COUNT;i++) {
 						if (ciphers_name[i]!=NULL) {
-							fprintf(stderr, "%i -  %s\n",i,ciphers_name[i]);
+							PRINT_LINE; fprintf(stderr, "dact: %3i | %s\n",i,ciphers_name[i]);
 						}
 					}
-					return(1);
+					return(0);
 				}
 				for (i=0;i<CIPHER_COUNT;i++) {
 					if (ciphers_name[i]!=NULL) {
@@ -1080,8 +1170,13 @@ int main(int argc, char **argv) {
 				}
 				if (i==CIPHER_COUNT) i=-1;
 				ciphernum=i;
+				if (ciphernum==-1) {
+					PRINTERR("No such cipher.");
+					return(-1);
+				}
 				break;
 			case 'l':
+				PRINT_LINE; fprintf(stderr, "dact: Num | Name\n");
 				for (i=0;i<255;i++) {
 					if (algorithms[i]==NULL || algorithms[i]==DACT_FAILED_ALGO) continue;
 					PRINT_LINE; fprintf(stderr, "dact: %3i | %s\n",i,algorithm_names[i]);
@@ -1163,8 +1258,10 @@ int main(int argc, char **argv) {
 		in_file=in_files[filecnt];
 		if (in_file!=NULL) {
 /* Determine resulting file name */
-			out_file=dact_getoutfilename(in_file,mode);
-			if ((in_fd=open_net(in_file,O_RDONLY))==-1) {
+			if (out_file==NULL) out_file=dact_getoutfilename(in_file,mode);
+			if (!strcmp("-",in_file)) {
+				in_fd=STDIN_FILENO;
+			} else if ((in_fd=open_net(in_file,O_RDONLY))==-1) {
 				fprintf(stderr, "dact: Can't open %s.\n",in_file);
 				PERROR("open");
 				continue;
@@ -1196,7 +1293,7 @@ int main(int argc, char **argv) {
 			if (options[DACT_OPT_STDOUT]) out_fd=STDOUT_FILENO;
 		}
 
-/* Use STDIN/STDOUT if no files specified ... */
+/* Use STDIN/STDOUT if no files specified ... unless an outfile was specified... */
 		if (in_file==NULL && filecnt==0) {
 /* ... But only if STDOUT isn't a terminal */
 			if (isatty(STDOUT_FILENO) && options[DACT_OPT_FORCE]==0) {
