@@ -1,3 +1,23 @@
+/*
+ * Copyright (C) 2001  Roy Keene
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ *      email: dact@rkeene.org
+ */
+
 #ifndef _DACT_H
 #define _DACT_H
 
@@ -13,7 +33,7 @@
 
 #define DACT_VER_MAJOR 0
 #define DACT_VER_MINOR 8
-#define DACT_VER_REVISION 3
+#define DACT_VER_REVISION 9
 #ifdef DEBUG
 #define DACT_VER_SUB	"dev"
 #else
@@ -129,4 +149,8 @@ extern uint32_t DACT_BLK_SIZE;
 #define NO_NETWORK 1
 #endif
 
+#ifdef HAVE_DEV_URANDOM
+#define RANDOM_DEV "/dev/urandom"
+#endif
+
 #endif/*_DACT_H*/
